/*
  Remduino.cpp - Library for communication with REMLab
  Released into the public domain.
*/

#include "Arduino.h"
#include "Remduino.h"

Remduino::Remduino()
{
	for (int i = 0; i < REM_MAXDEV; i++){
			_controlBinds[i] = -1;
			_controlFuncs[i] = NULL;
	}

	_measureCount = 0;
	_sendCount = 0;
	
	_synced = false;
	_halfSynced = false;
}

void Remduino::bindControl(int pin,int device)
{
	_controlBinds[device] = pin;
}

void Remduino::bindControlFunction(int device,void (*f)(unsigned int input))
{
	_controlFuncs[device] = f;
}

void Remduino::bindMeasure(int pin,int device,bool analog)
{
	bindMeasure(pin,device,analog,NULL);
}

void Remduino::bindMeasure(int pin, int device, bool analog, unsigned int (*f)(unsigned int input))
{		
	_measureBinds[_measureCount].pinNum = pin;
	_measureBinds[_measureCount].deviceNum = device;
	_measureBinds[_measureCount].analog = analog;
	_measureBinds[_measureCount].func = f;
	_measureCount++;
}



void Remduino::sendValue(int device,unsigned int value)
{
	_sendQueue[_sendCount].deviceNum = device;
	_sendQueue[_sendCount].value = value;
	if (_sendCount < REM_SEND_QUEUE_LEN - 1){ _sendCount++; }
}

void Remduino::unbindControl(int device){
	if (device >= REM_MAXDEV) return;
	_controlBinds[device] = -1;
}

void Remduino::unbindControlFunction(int device){
	if (device >= REM_MAXDEV) return;
	_controlFuncs[device] = NULL;
}

void Remduino::unbindMeasure(int pin){
	for (int i = 0; i<_measureCount; i++){
		if (_measureBinds[i].pinNum == pin)
		{
			_measureBinds[i] = _measureBinds[_measureCount-1];
			_measureCount--;
		}
	}
}

void Remduino::process()
{
	byte readByte[3];
	
	while (true)
	{	
		if (_synced)
		{
			if (Serial.available() < 3) break;
			
			Serial.readBytes(readByte,3);
			
			if (readByte[0] >= REM_MAXDEV) continue;
			
			int pinNum = _controlBinds[readByte[0]];
			void (*func)(unsigned int input) = _controlFuncs[readByte[0]];
			
			unsigned int value = readByte[1]+(readByte[2]<<8);
			value = constrain(value,0,65534);
			unsigned int mappedValue = map(value,0,65534,0,255);
			
			if (pinNum != -1) 
			{
				analogWrite(pinNum,mappedValue);
			}
			
			if (func != NULL)
			{
				func(value);
			}
			//Serial.print(pinNum);
			//Serial.print("\t");
			//Serial.print(value);
			//Serial.print("\n");
			
			_synced = false;
			_halfSynced = false;
		}
		else
		{
			if (Serial.available() == 0) break;
			
			if (Serial.read() != REM_DIV) 
			{
				_synced = false;
				_halfSynced = false;
				continue;
			}
			
			if (_halfSynced)
			{
				_synced = true;
			}
			else
			{
				_halfSynced = true;
			}
		}
	}
	
	for (int i = 0; i < _measureCount; i++)
	{
		unsigned int value;
		
		if (_measureBinds[i].analog)
		{
			value = analogRead(_measureBinds[i].pinNum);
			value = map(value,0,1023,0,65534);
		}
		else
		{
			value = digitalRead(_measureBinds[i].pinNum);
			value = value*65534;
		}
		
		if (_measureBinds[i].func!=NULL)
		{
			value = _measureBinds[i].func(value);
		}
		
		Serial.write(REM_DIV);
		Serial.write(REM_DIV);
		Serial.write(_measureBinds[i].deviceNum);
		Serial.write(byte(value & 0xFF));
		Serial.write(byte(value>>8));
	}
	
	for (int i = 0; i < _sendCount; i++)
	{
		Serial.write(REM_DIV);
		Serial.write(REM_DIV);
		Serial.write(_sendQueue[i].deviceNum);
		Serial.write(byte(_sendQueue[i].value & 0xFF));
		Serial.write(byte(_sendQueue[i].value>>8));
	}
	
	_sendCount = 0;
}
